#!/bin/sh

#set -x

# This script will Start the patching process on the Primary Controller.


export PATH=/control4/bin:/usr/sbin:/bin:/usr/bin:/sbin
export LD_LIBRARY_PATH=/control4/lib:/opt/lib


SHT_NAME="C4_2100_291_Napster_Patch";
SCRIPT_VERSION="2.10.0.0002";
TIME_HOUR="03";
TIME_MINUTES="42";

GET_FILE="http://update.control4.com/patches/${SHT_NAME}";
P_VER=$(basename ${0} | cut -d '.' -f1 | cut -d '-' -f2);
PATCH_NAME="patch-napster";
#PATCH_VER="1.*.${P_VER}";
ARCH="all";

ST_NAME="fix-${P_VER}.sh"
P_SCRIPT="patchscript-${P_VER}"
V_SCRIPT="validate-${P_VER}"
D_SCRIPT="dlist-${P_VER}.sh"
IP_SCRIPT="ip-${P_VER}"
D_FILE="${PATCH_NAME}_*.${P_VER}_${ARCH}.deb"
MOD_NAME="pdl-${P_VER}.tgz"    
FINSPECT="fix-inspect.sh";

ETC="/etc";
C4="/control4";
C4BIN="${C4}/bin";
LOGDIR="/var/log";
MNTINT="/mnt/internal";

EMAIL_STATUS="${LOGDIR}/emailmessage.txt";
PALOG="${LOGDIR}/patch_applied";
PELOG="${LOGDIR}/patch_error";
CRONFILE="${ETC}/crontabs/root";

DPKGL="dpkg -l";
UNTAR="tar xzf ";
KILLALL="killall -9";
MD5="md5sum"; 
CHMOD="chmod +x ";
WGET="wget ";

GVERSION=0
NeedToDownload=0;
IsGood=0;    


for i in $*; do
    arg=$(echo ${i} | cut -d'=' -f1 | sed 's/-//g');
    value=$(echo ${i} | cut -d'=' -f2);
    eval export ${arg}=${value}
done


UpdateMessage()
{
    # $1 is the message string to update the Email File
    # $2 is the message file to write

    echo -e "-- $1 --" | tee -a $2    
}

AllPatched()
{
    UpdateMessage "Completed: $1"
    echo -e "Script: ${ST_NAME} Version: ${SCRIPT_VERSION}\n\n" > ${PALOG}
    echo -e "Completed: $1" >> ${PALOG}
        
    if [ ${GVERSION} -eq 0 ] ; then
        RemoveCrontab
    fi
    
    CleanOnExit 2
}

die()
{
    UpdateMessage "Error: ${1} " >> ${PELOG}
    # clean up and exit gracefully
    
    RemoveCrontab
    CleanOnExit ${2}
}


RemoveOldPatchFiles()
{
    rm -f patch-*.deb* validate-*. dlist-*.sh patchscript-* ${EMAIL_STATUS} ${PALOG} ${PELOG} ${LOGDIR}/${FINSPECT} &
}

CleanOnExit()
{
    # $1 is for the exit code

    rm -f ${LOGDIR}/${ST_NAME} ${LOGDIR}/${P_SCRIPT}* ${LOGDIR}/${MOD_NAME}* ${LOGDIR}/*${P_VER}* ${LOGDIR}/${FINSPECT} ${LOGDIR}/${D_FILE}*
    
    exit ${1}
}

CheckAndSetupCron()
{
   UpdateMessage "Updating the ${CRONFILE} "  ${EMAIL_STATUS}
   cp ${CRONFILE} ${CRONFILE}.${P_VER}
   echo "${TIME_MINUTES}  ${TIME_HOUR}  *   *   *  ${MNTINT}/${ST_NAME} dn=0;" >> ${CRONFILE}
   ${KILLALL} crond
}

CleanMNT()
{
    #UpdateMessage "Removing all the script files from ${MNTINT} "  ${EMAIL_STATUS}
    rm -f ${MNTINT}/${ST_NAME} ${MNTINT}/${P_SCRIPT}* ${MNTINT}/${MOD_NAME}* ${MNTINT}/${FINSPECT} ${MNTINT}/${D_FILE}*
}

RemoveCrontab()
{
    #UpdateMessage "Restoring the original ${CRONFILE} "  ${EMAIL_STATUS}
    
    grep -v "${ST_NAME}" ${CRONFILE} > ${CRONFILE}.fix-${P_VER}
    RESTORE=$(cat ${CRONFILE}.fix-${P_VER}); 
    if [ -n "${RESTORE}" ] ; then
        #UpdateMessage  "Cleaning up the Cron File and removing ${ST_NAME}."  ${EMAIL_STATUS}    
        sed -i /${ST_NAME}/d ${CRONFILE}
        rm -f ${CRONFILE}.fix-${P_VER} ${CRONFILE}.${P_VER}
    else
        UpdateMessage  "Error: The Temp Cron File was completely empty on ${ST_NAME}."  ${EMAIL_STATUS}    
    fi
    
    ${KILLALL} crond
    CleanMNT
}

CheckFileDownloaded()
{
    if [ ! -e ${LOGDIR}/${1} ] && [ ! -e ${MNTINT}/${1} ]; then
        NeedToDownload=1;
        return;
    fi
   
    if [ -e ${LOGDIR}/${1} ] && [ ! -e ${MNTINT}/${1} ] ; then
        cp ${LOGDIR}/${1} ${MNTINT}/${1}    
        cp ${LOGDIR}/${1}.${MD5} ${MNTINT}/${1}.${MD5}
    fi
}

SetupCronFirst()   
{   
   UpdateMessage "Setting up the ${SHT_NAME} program script in ${MNTINT}/${ST_NAME}."  ${EMAIL_STATUS}
   if [ ! -e "${MNTINT}/${ST_NAME}" ] ; then 
           cp ${LOGDIR}/${ST_NAME} ${MNTINT}/${ST_NAME}
   fi
       
   IsInstalled=$(grep -i ${MNTINT}/${ST_NAME} ${CRONFILE});
   
   if [ -z "${IsInstalled}" ] ; then
           
       #############################################################
       #
       # We need to see if the fix- script is in the crontab
       #
       #############################################################
       
       UpdateMessage "Adding a setting in the Crontab file."  ${EMAIL_STATUS}
       CheckAndSetupCron
   fi
}        

Download()
{   

   # $1 is the file name
   # $2 is the location of where to untar the files
    
    #############################################################
    #
    #   If we are waking up from a delayed install we don't need 
    #   to wget the file or check /var/log md5sum information.
    #
    #############################################################
    
    IsInstalled=$(grep -i ${MNTINT}/${ST_NAME} ${CRONFILE});
    # We need to see if the program is installed if not we need to clean up.
    if [ -z "${IsInstalled}" ] ; then
        CleanMNT
    fi
    
    if [ ! -e ${MNTINT}/${1} ] ; then
        
        cd ${2}
        CheckFileDownloaded ${1}
        
        if [ ${NeedToDownload} -eq 1 ] ; then   
            UpdateMessage "Downloading the following file ${WGET} ${GET_FILE}/${1} "  ${EMAIL_STATUS}
            ${WGET} ${GET_FILE}/${1}
            ${WGET} ${GET_FILE}/${1}.${MD5}
        fi
        
        MOD_NAME_MD5=$(${MD5} ${1} | cut -c 1-32);
        RECORDED_MD5=$(cat ${1}.${MD5} | cut -c 1-32);
    
        [ -s ${1} ] || die "${1} not found" "1"
        [ -s ${1}.${MD5} ] || die "${1}.${MD5} not found" "1"
        [ -z "${MOD_NAME_MD5}" ] && die "Could not get the ${MD5} from the downloaded file ${1}" "1"
        [ -z "${RECORDED_MD5}" ] && die "Could not get the ${MD5} from the downloaded file ${1}.${MD5}" "1"
        [ "${RECORDED_MD5}"x == "${MOD_NAME_MD5}"x ] || die "The file ${1} downloaded ${MD5} value didn't match the recorded ${MD5} value for the downloaded file ${MOD_NAME_MD5} and Recorded ${MD5} ${RECORDED_MD5}" "1"
    
        #############################################################
        #
        #   We have the files on the Master Controller so let's not download again.
        #   We need to save them on the /mnt/internal for later.
        #
        #############################################################
       
        if [ ${NeedToDownload} -eq 1 ] ; then   
            CheckFileDownloaded ${1}
        fi    
    fi   
}

StartUp_CheckTheVersion()
{

    # We need to check the uptime of the system
    MainUptime=$(uptime | grep -i days | cut -d ',' -f1 | cut -d ' ' -f4);
    
    if [ -z "${MainUptime}" ] ; then
       MainUptime=0;
    fi

    # Get the version of the Controller
    Get_Cntl_Ver=$(${DPKGL} | grep system-version | tr -s ' '| cut -d ' ' -f3);
    RH_Maj=$(echo ${Get_Cntl_Ver} | tr -s ' '| cut -d ' ' -f3 | cut -d '.' -f1);
    RH_Min=$(echo ${Get_Cntl_Ver} | tr -s ' '| cut -d ' ' -f3 | cut -d '.' -f2);
    RH_Rev=$(echo ${Get_Cntl_Ver} | tr -s ' '| cut -d ' ' -f3 | cut -d '.' -f3);
    RH_Bld=$(echo ${Get_Cntl_Ver} | tr -s ' '| cut -d ' ' -f3 | cut -d '-' -f1 | cut -d '.' -f4);
                

    # Get the Host Processor Type
    DEVICE_TYPE=$(uname -m);

    # Get the Host Controller Name
    CN_NAME=$(uname -a | cut -d' ' -f2);
    DEVICE_NAME=$(uname -v | cut -d' ' -f1 | cut -d '-' -f2 | cut -d '.' -f1);
    #DEVICE_MAC=$(ifconfig | grep -i 'eth0' | cut -d 'W' -f2 | cut -d' ' -f2 );
    DEVICE_MAC=$(cat /etc/smb.conf | grep -i netbios | cut -d '=' -f2 | cut -d '-' -f1 | tr -s ' ');

    [ -z "${dn}" ] && dn=1
    [ -n "${cn}" ] && CN_NAME=${cn}

    if [ ${dn} -eq 2 ] ; then
        UpdateMessage "Script Name = ${ST_NAME}"  ${EMAIL_STATUS}
        UpdateMessage "Script Version = ${SCRIPT_VERSION}"  ${EMAIL_STATUS}
        UpdateMessage "Common Name = ${CN_NAME}"  ${EMAIL_STATUS}
        UpdateMessage "Home Controller Version = ${Get_Cntl_Ver}"  ${EMAIL_STATUS}
        UpdateMessage "Host MAC = ${DEVICE_MAC}"  ${EMAIL_STATUS}
        UpdateMessage "Device Type = ${DEVICE_TYPE}"  ${EMAIL_STATUS}
        UpdateMessage "Device Name = ${DEVICE_NAME}"  ${EMAIL_STATUS}
        UpdateMessage "Controller uptime in Days = ${MainUptime}"  ${EMAIL_STATUS}
    fi

    IsGood=0;    
    if [ ${RH_Maj} -eq 2 ] && [ ${RH_Min} -eq 9 ]  && [ ${RH_Rev} -eq 1 ] ; then
        IsGood=1;    
    elif [ ${RH_Maj} -eq 2 ] && [ ${RH_Min} -eq 10 ] && [ ${RH_Rev} -eq 0 ] ; then
        IsGood=1;    
    fi
    
    if [ ${IsGood} -eq 1 ]; then    
       #UpdateMessage "This OS is the correct version for the ${SHT_NAME} Program."  ${EMAIL_STATUS}
       GVERSION=1
    else
       die "The Major.Minor.Rev version # is not a valid version number to run the ${SHT_NAME} Program." "1"
    fi
    
}

#############################################################
#  MAIN of the Script
#############################################################

cd ${LOGDIR}

#############################################################
# Cleaning up the old files on the system
#############################################################
RemoveOldPatchFiles

#############################################################
# Lets Startup and Check the version of the system before we do anything
#############################################################
StartUp_CheckTheVersion

#############################################################
# We are limited on space on old controllers so we need to clean up.
#############################################################
rm -f ${LOGDIR}/debug/*.gz    
    
#############################################################
# We need to download the files from the update server
# If the files have not already been downloaded
#############################################################
Download ${MOD_NAME} ${LOGDIR}

#############################################################
# Delaying the patch install until later
#############################################################

if [ ${dn} -eq 1 ] ; then
    
    #############################################################
    # Lets Check the version of the system before we do anything
    #############################################################
    SetupCronFirst
    UpdateMessage "Delaying the deployment of the patch on the system until ${TIME_HOUR}:${TIME_MINUTES} AM."  ${EMAIL_STATUS}
    CleanOnExit 3;

elif [ ${dn} -eq 3 ] ; then
    
    #############################################################
    # Windows Patch Install and we need to clean up .
    #############################################################
    RemoveCrontab
    CleanOnExit 2;

elif [ ${dn} -eq 0 ] ; then
    UpdateMessage "Crontab Patch Deploy on the system Now."  ${EMAIL_STATUS}
    
    #############################################################
    # Untarring the downloaded file
    #############################################################
   
    UpdateMessage "UnTarring the downloaded file ${1}"  ${EMAIL_STATUS}
    ${UNTAR} ${MNTINT}/${MOD_NAME} -C ${LOGDIR}
    
    echo "---------------------------------------------------------------------------------";
    ${CHMOD} ${LOGDIR}/${P_SCRIPT}* 
    ${LOGDIR}/${P_SCRIPT} vd=1
    RETVAL=$(echo $?);
    
    echo "---------------------------------------------------------------------------------";
    
    if [ "${RETVAL}" == "2" ] ; then
        AllPatched "The system already has the patch." ${EMAIL_STATUS}
        
    elif [ "${RETVAL}" == "4" ] ; then 
        die "No Controller Devices in the project" "4"
    elif [ "${RETVAL}" == "1" ] ; then 
        die "We received an error from the ${LOGDIR}/${P_SCRIPT}" "1"
    else
        UpdateMessage "Not all devices are patched on the system Validate Status = ${RETVAL} " ${EMAIL_STATUS}
    fi
        
    echo "---------------------------------------------------------------------------------";
    
    ${CHMOD} ${LOGDIR}/${P_SCRIPT}* 
    ${LOGDIR}/${P_SCRIPT} pr=0
    RETVAL=$(echo $?);
    
    echo "---------------------------------------------------------------------------------";
    
    if [ "${RETVAL}" == "2" ] ; then
        AllPatched "The system already has the patch." ${EMAIL_STATUS}
        
    elif [ "${RETVAL}" == "1" ] ; then 
        die "We received an error from the ${LOGDIR}/${P_SCRIPT}" "1"
    else
        UpdateMessage "Completed the patch deployment to all devices Status = ${RETVAL} " ${EMAIL_STATUS}
    fi
    
    echo "---------------------------------------------------------------------------------";
    
    #REBOOTME=$(grep -i 127\.0\.0\.1 ${LOGDIR}/ip-${P_VER} | wc -l | tr -d '\n';);
    #UpdateMessage "Checking to see if we need to reboot the Primary Device = ${REBOOTME} " ${EMAIL_STATUS}
    #
    #echo "---------------------------------------------------------------------------------";
    
    ${CHMOD} ${LOGDIR}/${P_SCRIPT}* 
    ${LOGDIR}/${P_SCRIPT} pr=1;
    RETVAL=$(echo $?);
    
    echo "---------------------------------------------------------------------------------";
    
    
    #echo "---------------------------------------------------------------------------------";
    ## We need to reboot all the devices in the IP List
    #${CHMOD} ${LOGDIR}/${P_SCRIPT}* 
    #${LOGDIR}/${P_SCRIPT} rb=1;
    ##RETVAL=$(echo $?);
    #echo "---------------------------------------------------------------------------------";

    #if [ ${REBOOTME} -eq 1 ] ; then
    #    (sleep 10 && reboot ) &
    #fi
    
    if [ "${RETVAL}" == "2" ] ; then
        AllPatched "The system already has the patch." ${EMAIL_STATUS}
        
    elif [ "${RETVAL}" == "1" ] ; then 
        die "We received an error from the ${LOGDIR}/${P_SCRIPT}" "1"
    else
        UpdateMessage "Completed the patch deployment to all devices Status = ${RETVAL} " ${EMAIL_STATUS}
    
        #############################################################
        # we need to clean up.
        #############################################################
        RemoveCrontab
    fi
    
    CleanOnExit 0;

else
#elif [ ${dn} -eq 2 ] ; then
    #############################################################
    #
    # Windows Wrapper Interface deployment.
    #
    #############################################################
    UpdateMessage "UnTarring the downloaded file ${1}"  ${EMAIL_STATUS}
    ${UNTAR} ${MNTINT}/${MOD_NAME} -C ${LOGDIR}
    exit;
fi


