#!/bin/sh

#set -x

export PATH=/control4/bin:/usr/sbin:/bin:/usr/bin:/sbin;
export LD_LIBRARY_PATH=/control4/lib:/opt/lib;

# This script will see if the device is a Controller Type

P_Label="C4 Patch device List Validator";
SCRIPT_VERSION="2.10.0.00001";
DEVICE_LIST="";

LOGDIR="/var/log";
EMAIL_STATUS="${LOGDIR}/emailmessage.txt";
DEVICE_SUPPORTED=0;
IsAlive=0;
SMAN="/control4/bin/sysman ";
IsGood=0;

for i in $*; do
    arg=$(echo ${i} | cut -d'=' -f1 | sed 's/-//g')
    value=$(echo ${i} | cut -d'=' -f2)
    eval export ${arg}=${value}
done

UpdateMessage()
{
    # $1 is the message string to update the Email File
    # $2 is the message file to write
    echo -e "${1}" | tee -a ${2}    
}

CleanOnExit()
{
    # $1 is for the exit code
    # We need to do some house cleaning   
    exit ${1}
}

die()
{
    #UpdateMessage "Error: ${1} " >> ${LOGDIR}/patch_error;
    # clean up and exit gracefully
    CleanOnExit 1
}

InstallToDevice()
{
    IsGood=0;
    
    if [ ! "${1}" == "127.0.0.1" ] ; then
        IsAlive=$(ping -q -c 4 $1 > /dev/null 2> /dev/null; echo $? | tr -d '\n');
    fi
    
    if [ ${IsAlive} -eq 0 ] ; then
        # Connection succeeded, querying sysmand.
        
        if [ "${1}" == "127.0.0.1" ]; then
            SINFO=$(dpkg -l);
        else
            SINFO=$(${SMAN} -h ${1} version);
        fi
    
        RH_Maj=$(echo "${SINFO}" | grep -i 'system-version' | grep -v 'grep' | tr -s ' ' | cut -d ' ' -f3 | cut -d '.' -f1 );
        RH_Min=$(echo "${SINFO}" | grep -i 'system-version' | grep -v 'grep' | tr -s ' ' | cut -d ' ' -f3 | cut -d '.' -f2 );
        RH_Rev=$(echo "${SINFO}" | grep -i 'system-version' | grep -v 'grep' | tr -s ' ' | cut -d ' ' -f3 | cut -d '.' -f3 );
        RH_Bld=$(echo "${SINFO}" | grep -i 'system-version' | grep -v 'grep' | tr -s ' ' | cut -d ' ' -f3 | cut -d '-' -f1 | cut -d '.' -f4);
        
        if [ "${RH_Maj}" == "" ] ; then
            die "The Device is NOT a Control4 controller.." ${EMAIL_STATUS}
        fi

        IsGood=0;
        if [ ${RH_Maj} -eq 2 ] && [ ${RH_Min} -eq 9 ]  && [ ${RH_Rev} -eq 1 ] ; then
            IsGood=1;    
        elif [ ${RH_Maj} -eq 2 ] && [ ${RH_Min} -eq 10 ] && [ ${RH_Rev} -eq 0 ] ; then
            IsGood=1;    
        fi
        
        if [ ${IsGood} -eq 1 ]; then    

            if [ "${1}" == "127.0.0.1" ]; then
                device_uname=$(uname -a | grep -v 'grep' | cut -d '#' -f2 | cut -d ' ' -f1 | cut -d '-' -f2;);
            else
                device_uname=$(${SMAN} -h ${1} sysinfo | grep -v 'grep' | grep 'uname' | cut -d '#' -f2 | cut -d ' ' -f1 | cut -d '-' -f2;);
            fi

            
            if [ -n "${device_uname}" ] ; then
                case "${device_uname}" in
                    hc250*|hc800*|hc1000*|ninjago*)
                        DEVICE_SUPPORTED=1
                    ;;
                esac
                    
                if [ ${DEVICE_SUPPORTED} -eq 1 ] ; then
                    UpdateMessage "Controller=$1 "  ${EMAIL_STATUS}                
                else
                    die "The Device does not qualify for this patch at IP Address $1 on the ${device_uname}."
                fi
            else
                die "The Device Name was not returned from this IP Address $1 "
            fi
        else
            die "The Device is NOT running the correct OS Version for this Patch." ${EMAIL_STATUS}
        fi

    else
        die "The Device at at IPAddress ${1} does not respond to a ping."
    fi
}

##################################
# Perform the patching operation #
##################################

[ -z "${ip}" ] && die "No IP Address Passed in."  ${EMAIL_STATUS}

InstallToDevice ${ip}
CleanOnExit 0
    
