#!/bin/sh

#set -x


# This script is the main patch which updates the inspector files on the device after it's been patched.

export PATH=/control4/bin:/usr/sbin:/bin:/usr/bin:/sbin
export LD_LIBRARY_PATH=/control4/lib:/opt/lib

# Get the Host Processor Type
DEVICE_TYPE=$(uname -m);

ETC="/etc";
C4BIN="/control4/bin";
MNTI="/mnt/internal";
MD5_DIR="/var/lib/dpkg/info";
MD5SUM="/usr/bin/md5sum";
LOGDIR="/var/log";
EMAIL_STATUS="${LOGDIR}/emailmessage.txt";

TCH="touch";
MOUNT="mount / -o remount";
DNULL="/dev/null";
PTYPE=0;
PATCH_DIR="";
GPATCH_DIR="";
PATCH_FILE="";
FNAME="";
FNAME_CNT=0;
DUP=0;

Run_Main()
{
    case $1 in
        -u|-U)
            PTYPE=0;
            ;;
        -n|-N)
            PTYPE=0;
            DUP=1;
            ;;
        -r|-R)
            PTYPE=1;
            ;;
        -c|-C)
            PTYPE=2;
            ;;
        -d|-D)
            PTYPE=3;
            ;;
        -a|-A)
            PTYPE=4;
            ;;
        -p|-P)
            PTYPE=5;
            ;;
        -s|-S)
            PTYPE=6;
            ;;
        *)
            Usage;
            ;;
    esac

    
    if [ ${PTYPE} -lt 4 ] && [ -n "${2}" ]; then
        PATCH_DIR=$(dirname "${2}");
        GPATCH_DIR=$(dirname "${2}" | cut -c 2-);
        PATCH_FILE=$(basename "${2}" );
    
    elif [ ${PTYPE} -eq 4 ] && [ -n "${3}" ]; then
        PATCH_DIR=$(dirname "${2}");
        GPATCH_DIR=$(dirname "${2}" | cut -c 2-);
        PATCH_FILE=$(basename "${2}");
        
        MD5_COUNT=$(echo "${3}" | wc -c | tr -d '\n' );
        NEW_MD5="${3}";
        
        if [ ! "${MD5_COUNT}" == "32" ]; then
            Usage;
        fi
        
    elif [ ${PTYPE} -eq 5 ] && [ -n "${2}" ]; then
        PATCH_DIR=$(dirname "${2}");
        GPATCH_MD5=$(cat "${2}.md5sum" | cut -c 1-32);
        PATCH_FILE=$(basename "${2}" );
    
    elif [ ${PTYPE} -eq 6 ] && [ -n "${2}" ]; then
        PATCH_FILE="${2}";
    else
        Usage;
    fi

    if [ ${PTYPE} -lt 5 ] ; then
        UpdateMessage "Getting a count of ${PATCH_FILE} in the following directory ${MD5_DIR}." ${EMAIL_STATUS}
        FNAME_CNT=$(grep -i "${GPATCH_DIR}/${PATCH_FILE}"$ ${MD5_DIR}/*.md5sums | grep -iv patch- | cut -d ':' -f1 | cut -d ' ' -f1 | wc -l);
    
        UpdateMessage "Looking for the ${PATCH_FILE} in the following directory ${MD5_DIR}." ${EMAIL_STATUS}
        FNAME=$(grep -i "${GPATCH_DIR}/${PATCH_FILE}"$ ${MD5_DIR}/*.md5sums | grep -iv patch- | cut -d ':' -f1 | cut -d ' ' -f1);
    
        if [ ! ${PTYPE} -eq 3 ] ; then
            if [ ${FNAME_CNT} -gt 1 ]; then
               UpdateMessage "Note: We have more then 1 file to fix. We need to loop on the count = ${FNAME_CNT}."  ${EMAIL_STATUS}
            fi
            
            if [ -z "${FNAME}" ] && [ "${GPATCH_DIR}" != "packages" ]; then
               UpdateMessage "Note: We Do not need to patch the inspector files in the following directory ${MD5_DIR}."  ${EMAIL_STATUS}
               DUP=1;
               CleanOnExit 0;
            fi
        else
            UpdateMessage "Getting for the ${PATCH_FILE} information." ${EMAIL_STATUS}
            FNAME=$(echo ${PATCH_FILE} | cut -d '_' -f1 | tr -d '\n');                                                   
        fi
    fi 
    
    if [ ${PTYPE} -eq 0 ]; then
        UpDateDevice "${FNAME}" "${PATCH_DIR}" "${PATCH_FILE}" "${GPATCH_DIR}"
    elif [ ${PTYPE} -eq 1 ]; then
        
        CNT=1;
        while [ ${CNT} -le ${FNAME_CNT} ] ; do
    
            R_LIST=$(echo "${FNAME}" | cut -d ' ' -f${CNT});
            Remove_File_From_MD5SUM "${R_LIST}" "${PATCH_FILE}" "${GPATCH_DIR}"
            CNT=$(( ${CNT} + 1 ))
    
        done 

        
    elif [ ${PTYPE} -eq 2 ]; then
        isApplied_Correctly "${FNAME}" "${PATCH_DIR}" "${PATCH_FILE}" "${GPATCH_DIR}"   
    elif [ ${PTYPE} -eq 3 ]; then
        File_Update_Info_File "${FNAME}" "${PATCH_FILE}" "${GPATCH_DIR}"
    elif [ ${PTYPE} -eq 4 ]; then
        JustAddPassedInMD5 "${FNAME}" "${PATCH_FILE}" "${GPATCH_DIR}" ${NEW_MD5}   
    elif [ ${PTYPE} -eq 5 ]; then
        Update_Info_File "${PATCH_DIR}" "${PATCH_FILE}" "${GPATCH_MD5}"
    elif [ ${PTYPE} -eq 6 ]; then
        Remove_Info "${PATCH_FILE}"
    else
        Usage;
    fi
    
}

Usage()
{
    clear
    echo "";
    echo "Not the correct arguments passed in.";
    echo "";
    echo "";
    echo " Usage: ";
    echo "";
    echo " (ADD MD5)              $0 -a /control4/bin/testfile.txt 0cb79a26529603886f1b7e58c3eeabd1";
    echo " (Check)                $0 -c /control4/bin/testfile.txt";
    echo " (fix update-info file) $0 -d /packages/c4-node-mods_2.6.0.455411-res.deb";
    echo " (Update No Root-dup)   $0 -n /control4/bin/testfile.txt";
    echo " (Remove)               $0 -r /control4/bin/testfile.txt";
    echo " (Update)               $0 -u /control4/bin/testfile.txt";
    echo " (Adding Patch Info)    $0 -p /var/log/patchname_version_all.deb";
    echo " (Remove Patch Info)    $0 -s patchname";
    echo "";
    echo "";
    CleanOnExit 3;
}

File_Update_Info_File()
{
   
   #echo $1  The File Name passed in
   #echo $2  The new Package Name passed in
   
   DTYPE=$(uname -v | cut -d' ' -f1 | cut -d '-' -f2 | cut -d '.' -f1 | tr -d '\n');
   FN=$(echo "${2}" | cut -d '_' -f1 | tr -d '\n');
   VN=$(echo "${2}" | cut -d '_' -f2 | tr -d '\n');
   U_PATH="/${3}/${DTYPE}";
   UPD_INFO=$(ls ${U_PATH}/update-info* | wc -l | tr -d '\n');
   
   if [ ${UPD_INFO} -eq 0 ]; then
       die "We did not find the ${U_PATH}/update-info*.";
   fi
   
   UPD_INFO=$(basename $(ls ${U_PATH}/update-info* | tr -d '\n'));
   
   OLDINFO=$(grep -A3 filename.${FN} ${U_PATH}/${UPD_INFO});
   OLDFN=$(echo ${OLDINFO} | cut -d ">" -f2 | cut -d "<" -f1 | tr -d '\n');
   OLDFS=$(echo ${OLDINFO} | cut -d ">" -f4 | cut -d "<" -f1 | tr -d '\n');
   OLDM5=$(echo ${OLDINFO} | cut -d ">" -f6 | cut -d "<" -f1 | tr -d '\n');
   LNMD5=${#OLDM5};
   
   if [ -z "${OLDFN}" ] ; then
       die "Error:  We Did not get the filename from the file ${U_PATH}/${UPD_INFO}.";
   fi
   
   if [ -z "${OLDFS}" ] && [ ${OLDFS} -le 0 ] ; then
       die "Error:  We Did not get the filesize from the file ${U_PATH}/${UPD_INFO}.";
   fi
   
   if [ -z "${OLDM5}"] && [ ! ${LNMD5} -eq 32 ] ; then
       die "Error:  We Did not get the md5sum value from the file ${U_PATH}/${UPD_INFO}.";
   fi
   
   NEWFN=${2};
   if [ -z "${NEWFN}" ] && [ ! -e "${NEWFN}" ]; then
       die "Error:  We Did not find the file ${2}.";
   fi
   
   NEWFS=$(ls -la "/${3}/${2}" | tr -s ' ' | cut -d ' ' -f5 | tr -d '\n');
   if [ -z "${NEWFS}" ] && [ ${OLDFS} -le 0 ] ; then
       die "Error:  We Did not get the filesize from the file ${2}.";
   fi
   
   NEWM5=$(md5sum "/${3}/${2}" | cut -c 1-32 | tr -d '\n');
   LNNM5=${#NEWM5};
   if [ -z "${NEWM5}"] && [ ! ${LNNM5} -eq 32 ] ; then
       die "Error:  We Did not get the md5sum value from the file ${2}.";
   fi
   
   # Lets sed the old values with the new values
   sed -i s/"${OLDFN}"/"${NEWFN}"/g  ${U_PATH}/${UPD_INFO}
   sed -i s/"${OLDFS}"/"${NEWFS}"/g  ${U_PATH}/${UPD_INFO}
   sed -i s/"${OLDM5}"/"${NEWM5}"/g  ${U_PATH}/${UPD_INFO}
   
}

genpackageinfo()
{
	FullPathName="$1/$2";
        FNameOnly=$(echo $2 | cut -d '_' -f1 | tr -d '\n');
        outfile="${MNTI}/patch.info";
        
        if [ -e ${FullPathName} ]; then
        
            UpgradeString=$(grep -i ${FNameOnly} ${4} | wc -l | tr -d '\n');
            if [ ${UpgradeString} -gt 3 ]; then
                Remove_Info "${FNameOnly}"
            fi
        
            size=$(stat -c %s ${FullPathName})
            csum=$3
    
            echo -e "\t\t<platform>patch</platform><--${FNameOnly}-->" >> $outfile
            echo -e "\t\t<name>${FNameOnly}</name><--${FNameOnly}-->" >> $outfile
            echo -e "\t\t<dpkinfo/><--${FNameOnly}-->" >> $outfile
            echo -e "\t\t<filename>$2</filename><--${FNameOnly}-->" >> $outfile
            echo -e "\t\t<filesize>${size}</filesize><--${FNameOnly}-->" >> $outfile
            echo -e "\t\t<md5sum>${csum}</md5sum><--${FNameOnly}-->" >> $outfile
            echo -e "\t\t<shutdownService/><--${FNameOnly}-->" >> $outfile
            echo -e "\t\t<loadable>false</loadable><--${FNameOnly}-->" >> $outfile
            echo -e "\t\t<description>${FNameOnly}</description><--${FNameOnly}-->" >> $outfile
            echo -e "\t</package><package><--${FNameOnly}-->" >> $outfile
            
            f1=$(cat $outfile);
            awk -vf1="$f1" '/<update-info>/{print;print f1;next}1' ${4} > $outfile
            mv $outfile ${4};
        fi
}

Remove_Info()
{
    SYS_VER=$(dpkg -l | grep -i system-version | tr -s ' ' | cut -d ' ' -f3 | tr -d '\n');
    UPD_INFO=$(ls ${MNTI}/update-info_${SYS_VER} | tr -d '\n');
    
    if [ -e ${UPD_INFO} ] && [ -n "${UPD_INFO}" ]; then
        sed -i /${1}/d "${UPD_INFO}"
    else
       die "We did not find the ${UPD_INFO} file.";
    fi
}

Update_Info_File()
{
    SYS_VER=$(dpkg -l | grep -i system-version | tr -s ' ' | cut -d ' ' -f3 | tr -d '\n');
    UPD_INFO=$(ls ${MNTI}/update-info_${SYS_VER} | tr -d '\n');
    
    if [ -e ${UPD_INFO} ] && [ -n "${UPD_INFO}" ]; then
    
        genpackageinfo "${1}" "${2}" "${3}" "${UPD_INFO}"
    else
       die "We did not find the ${UPD_INFO} file.";
    fi
}

UpDateDevice()
{
    
    UpdateMessage "Finding the Old MD5sum value from the file ${3}." ${EMAIL_STATUS}
    OLDMD5=$(grep -i "${4}/${3}"$ ${1} | cut -c 1-32);
    
    UpdateMessage "Finding the New MD5sum value from the file ${3}.." ${EMAIL_STATUS}
    NEWMD5=$($MD5SUM "${2}/${3}" | cut -c 1-32);
    
    
    if [ -z "${OLDMD5}" ] || [ -z "${NEWMD5}" ]; then
       die "We are missing one of the New or Old MD5sum values.";
    fi                                                   
                                                       
    if [ "${OLDMD5}" == "${NEWMD5}" ] ; then         
        AllPatched "The configuration change has already been applied correctly on the device."  ${EMAIL_STATUS}
    
    else
        
        UpdateMessage "Updating the MD5sums from ${OLDMD5} to ${NEWMD5} in the file ${1}." ${EMAIL_STATUS}
        sed -i s/${OLDMD5}/${NEWMD5}/g  ${1};
    
        ##########################################################        
        # We need to make sure the patch applied correctly       #
        ##########################################################
        isApplied_Correctly "${FNAME}" "${PATCH_DIR}" "${PATCH_FILE}" "${GPATCH_DIR}"
    
    fi
}    

Remove_File_From_MD5SUM()
{
    UpdateMessage "Finding the New MD5sum value from the file ${2}." ${EMAIL_STATUS}
    OLDMD5=$(grep -i "${3}/${2}"$ ${1} | cut -c 1-32);
    
    if [ -z "${OLDMD5}" ] ; then
       die "Error:  We are missing the Old MD5sum values in the file ${2}.";
    fi                                                   
                                                       
    UpdateMessage "Removing the file ${2} with the MD5SUM of ${OLDMD5} from ${1}." ${EMAIL_STATUS}
    sed -i /${OLDMD5}/d  ${1};
        
}
    
isApplied_Correctly()
{
    MD5INFO=$(grep -i "${4}/${3}"$ ${1} | cut -c 1-32);
    
    UpdateMessage "Finding the New MD5sum value from the file ${3}.." ${EMAIL_STATUS}
    NEWMD5=$($MD5SUM "${2}/${3}" | cut -c 1-32);
    
    echo "This is the MD5sum = ${MD5INFO} this is the new MD5sum = ${NEWMD5}";
    
    if [ "${MD5INFO}" == "${NEWMD5}" ] ; then         
        UpdateMessage "The configuration change for ${3} applied correctly to the file ${1} on the device."  ${EMAIL_STATUS}
    else
        die "Failed to apply the configuration change ${3} correctly to the file ${1} on the device."
    fi
    
}

JustAddPassedInMD5()
{ 
    #${FNAME} ${PATCH_FILE} ${GPATCH_DIR} ${NEW_MD5}   

    OLDMD5=$(grep -i "${3}/${2}"$ ${1} | cut -c 1-32);
    
    UpdateMessage "Using the passed in MD5sum value ${4}" ${EMAIL_STATUS}
    NEWMD5="${4}";
    
    
    if [ -z "${OLDMD5}" ]; then
       die "We couldn't find the OLD MD5sum value.";
    fi                                                   
                                                       
    if [ -z "${NEWMD5}" ]; then
       die "We are missing the Passed in MD5sum values.";
    fi                                                   
                                                       
    UpdateMessage "Updating the MD5sums to ${NEWMD5} in the file ${1}." ${EMAIL_STATUS}
    sed -i s/${OLDMD5}/${NEWMD5}/g  ${1};
    
}

AllPatched()
{
        PATCH_STATUS="patch:AllReadyApplied"
        UpdateMessage "Completed: $1"
        echo -e "Script: $SCRIPT_NAME Version: $SCRIPT_VERSION\n\n" > ${LOGDIR}/patch_applied
        echo -e "Completed: $1" >> ${LOGDIR}/patch_applied
        echo -e "Completed: $1" >> ${EMAIL_STATUS}
        
        CleanOnExit 2
}

UpdateMessage()
{
    # $1 is the message string to update the Email File
    # $2 is the message file to write

    echo -e "-- $1 --" | tee -a $2    
}

remount()
{
    if [ "${DEVICE_TYPE}" != "i686" ] && [ ${DUP} -eq 0 ]; then
        rm -f ${ETC}/rootdup.conf
        #UpdateMessage "Remounting the root file system read only for ${DEVICE_TYPE}."
        sync
        sleep 1
        
        ${MOUNT},ro
        UpdateMessage "Updating the secondary partition on ${DEVICE_TYPE}"
        ${C4BIN}/root-dup > ${DNULL}
    fi

    sync        
}

die()
{
    UpdateMessage "Error: ${1} "  ${LOGDIR}/patch_error
    # clean up and exit gracefully
    CleanOnExit 1
}

CleanOnExit()
{
    # $1 is for the exit code

    if [ ${1} -eq 0 ] ; then
        remount
    fi
    
    #rm -f ${LOGDIR}/${0} &
    
    exit ${1}

}

##########################################################        
# Main Function of this script                           #
##########################################################

Run_Main "$1" "$2" "$3"


##########################################################        
# Cleaning up and exiting the program                    #
##########################################################

CleanOnExit 0

